import { Alert, Button, Center, Divider, Heading, Icon, SimpleGrid, Spinner, Text, VStack } from "@chakra-ui/react";
import { useWallet } from "@solana/wallet-adapter-react";
import { useBondingPricing, useOwnedAmount, useReserveAmount, useTokenBondingFromMint, useTokenMetadata } from "@strata-foundation/react";
import moment from "moment";
import React, { useMemo, useState } from "react";
import { RiPencilFill } from "react-icons/ri";
import { useBountyInfo } from "../../hooks/useBountyInfo";
import { useIsBountyAdmin } from "../../hooks/useIsBountyAdmin";
import { AuthorityAndTokenInfo } from "./AuthorityAndTokenInfo";
import { BountyCardContribution } from "./BountyCardContribution";
import { BountyContribute } from "./BountyContribute";
import { DisburseFunds } from "../DisburseFunds";
import { TopHolders } from "./TopHolders";
export const BountyDetail = ({ name, description, image, mintKey, onEdit, onBuyMore }) => {
    const { info: tokenBonding, loading: bondingLoading } = useTokenBondingFromMint(mintKey);
    const { publicKey } = useWallet();
    const { pricing } = useBondingPricing(tokenBonding?.publicKey);
    const targetBalance = useOwnedAmount(tokenBonding?.targetMint);
    const reserveAmount = useReserveAmount(tokenBonding?.publicKey);
    const [topHolderKey, setTopHolderKey] = useState(0);
    const refreshTopHolders = () => setTopHolderKey((k) => k + 1);
    const { isAdmin, isEditor } = useIsBountyAdmin(publicKey || undefined, tokenBonding?.publicKey);
    const { info: { name: fromChainName, image: targetImage, createdAt, description: fromChainDescription, fundsHaveBeenUsed, bountyClosed, attributes, isNormalBounty, }, loading, } = useBountyInfo(mintKey);
    name = fromChainName || name;
    image = targetImage || image;
    description = fromChainDescription || description;
    const dataMissing = useMemo(() => !name && !image && !description, [name, image, description]);
    const { metadata: baseMetadata } = useTokenMetadata(tokenBonding?.baseMint);
    if ((!loading && dataMissing) ||
        (attributes && !attributes.is_strata_bounty)) {
        return (React.createElement(Center, { height: "300px" },
            React.createElement(Text, null,
                React.createElement("b", null, "404: "),
                " Not found")));
    }
    if (dataMissing) {
        return React.createElement(Spinner, null);
    }
    return (React.createElement(VStack, { spacing: 2, w: "full" },
        isEditor && (React.createElement(Button, { color: "gray.400", 
            // __hover={{ rounded: "lg", borderColor: "gray.200", backgroundColor: "gray.100" }}
            leftIcon: React.createElement(Icon, { as: RiPencilFill, mr: "-1px" }), variant: "ghost", marginLeft: "auto", onClick: () => onEdit && onEdit() }, "Edit")),
        React.createElement(VStack, { w: "full", p: 6, pt: isEditor ? 0 : 8, spacing: 8 },
            React.createElement(VStack, { spacing: 4 },
                React.createElement(Heading, { textAlign: "center" }, name),
                React.createElement(AuthorityAndTokenInfo, { mintKey: mintKey }),
                tokenBonding && (React.createElement(Text, { fontSize: "15px", color: "gray.400" },
                    "Created ",
                    moment(createdAt).fromNow()))),
            React.createElement(Text, { w: "full", align: "left", fontSize: "15px", color: "gray.500", whiteSpace: "pre-line" },
                description,
                "\n",
                attributes?.discussion && `Discussion: ${attributes.discussion}\n`,
                attributes?.contact && `Contact: ${attributes.contact}`),
            fundsHaveBeenUsed && (React.createElement(Alert, { status: "error" }, "Funds have been disbursed from this bounty without closing it. Existing contributors may not be able to withdraw what they put into the bounty. Contact the bounty authority if you have any questions")),
            !isNormalBounty && (React.createElement(Alert, { status: "warning" }, "This bounty does not have normal bonding curve parameters. It may have royalties set, or be using a non fixed price curve. Buyer beware.")),
            bountyClosed && (React.createElement(React.Fragment, null,
                React.createElement(Alert, { status: "info" }, "This bounty has been closed."),
                React.createElement(Divider, { color: "gray.200" }),
                React.createElement(Heading, { mb: "-6px", alignSelf: "flex-start", size: "sm" }, "Top Contributors"),
                React.createElement(TopHolders, { key: topHolderKey, mintKey: mintKey }))),
            !bountyClosed && (React.createElement(React.Fragment, null,
                React.createElement(SimpleGrid, { w: "full", justifyContent: "stretch", columns: [1, 1, 2], spacing: 2, gap: 2 },
                    React.createElement(BountyCardContribution, { amount: reserveAmount, symbol: baseMetadata?.data.symbol }),
                    React.createElement(BountyCardContribution, { amount: typeof targetBalance === "undefined"
                            ? undefined
                            : pricing?.sellTargetAmount(targetBalance), symbol: baseMetadata?.data.symbol, text: "My Contributions" })),
                React.createElement(VStack, { align: "flex-end", w: "full" },
                    React.createElement(BountyContribute, { onBuyMore: onBuyMore, mintKey: mintKey, onContributeSuccess: refreshTopHolders })),
                React.createElement(Divider, { color: "gray.200" }),
                React.createElement(Heading, { mb: "-6px", alignSelf: "flex-start", size: "sm" }, "Top Contributors"),
                React.createElement(TopHolders, { key: topHolderKey, mintKey: mintKey }),
                isAdmin && tokenBonding && (React.createElement(VStack, { w: "full", spacing: 2 },
                    React.createElement(Divider, { color: "gray.200" }),
                    React.createElement(Heading, { alignSelf: "flex-start", size: "sm" }, "Disburse Funds"),
                    React.createElement(DisburseFunds, { tokenBondingKey: tokenBonding?.publicKey, closeBonding: true }))))))));
};
//# sourceMappingURL=BountyDetail.js.map