import { Alert, Button, Center, Input, Spinner, VStack, } from "@chakra-ui/react";
import { yupResolver } from "@hookform/resolvers/yup";
import { DataV2, Metadata } from "@metaplex-foundation/mpl-token-metadata";
import { useWallet } from "@solana/wallet-adapter-react";
import { PublicKey } from "@solana/web3.js";
import { MarketplaceSdk } from "@strata-foundation/marketplace-sdk";
import { useProvider, useStrataSdks, useTokenBondingFromMint, useTokenMetadata, } from "@strata-foundation/react";
import { SplTokenBonding } from "@strata-foundation/spl-token-bonding";
import { SplTokenMetadata } from "@strata-foundation/spl-utils";
import { useRouter } from "next/router";
import React from "react";
import { useAsync, useAsyncCallback } from "react-async-hook";
import { FormProvider, useForm } from "react-hook-form";
import * as yup from "yup";
import { FormControlWithError } from "./FormControlWithError";
import { Recipient } from "./Recipient";
import { TokenMetadataInputs } from "./TokenMetadataInputs";
const validationSchema = yup.object({
    image: yup.mixed(),
    name: yup.string().required().min(2),
    description: yup.string(),
    shortName: yup.string().min(2).max(10),
    contact: yup.string(),
    discussion: yup.string(),
    authority: yup.string().required(),
});
async function editBounty(tokenMetadataSdk, tokenBondingSdk, values, mintKey) {
    const uri = await tokenMetadataSdk.uploadMetadata({
        provider: values.provider,
        name: values.name,
        symbol: values.shortName,
        description: values.description,
        image: values.image,
        mint: mintKey,
        attributes: MarketplaceSdk.bountyAttributes({
            mint: mintKey,
            discussion: values.discussion,
            contact: values.contact,
        }),
    });
    const instructions = [];
    const signers = [];
    const { instructions: metaInstrs, signers: metaSigners } = await tokenMetadataSdk.updateMetadataInstructions({
        metadata: await Metadata.getPDA(mintKey),
        data: new DataV2({
            // Max name len 32
            name: values.name.substring(0, 32),
            symbol: values.shortName.substring(0, 10),
            uri,
            sellerFeeBasisPoints: 0,
            creators: null,
            collection: null,
            uses: null,
        }),
    });
    instructions.push(...metaInstrs);
    signers.push(...metaSigners);
    const tokenBondingKey = (await SplTokenBonding.tokenBondingKey(mintKey))[0];
    const tokenBondingAcct = await tokenBondingSdk.getTokenBonding(tokenBondingKey);
    if (values.authority &&
        tokenBondingSdk.wallet.publicKey.equals(tokenBondingAcct.reserveAuthority)) {
        const authority = new PublicKey(values.authority);
        const { instructions: bondInstrs, signers: bondSigners } = await tokenBondingSdk.updateTokenBondingInstructions({
            tokenBonding: tokenBondingKey,
            generalAuthority: authority,
            reserveAuthority: authority,
        });
        instructions.push(...bondInstrs);
        signers.push(...bondSigners);
    }
    await tokenBondingSdk.sendInstructions(instructions, signers);
    return mintKey;
}
export const EditBountyFormRaw = ({ mintKey, values, onComplete, hide = new Set(), }) => {
    const formProps = useForm({
        resolver: yupResolver(validationSchema),
        defaultValues: values,
    });
    const { register, handleSubmit, watch, setValue, formState: { errors, isSubmitting }, } = formProps;
    const { publicKey } = useWallet();
    const { awaitingApproval } = useProvider();
    const { execute, loading, error } = useAsyncCallback(editBounty);
    const { tokenMetadataSdk, tokenBondingSdk } = useStrataSdks();
    const router = useRouter();
    const authority = watch("authority");
    const onSubmit = async (values) => {
        await execute(tokenMetadataSdk, tokenBondingSdk, values, mintKey);
        onComplete && onComplete();
    };
    const authorityRegister = register("authority");
    return (React.createElement(FormProvider, { ...formProps },
        React.createElement("form", { onSubmit: handleSubmit(onSubmit) },
            React.createElement(VStack, { spacing: 8 },
                React.createElement(TokenMetadataInputs, null),
                !hide.has("shortName") && (React.createElement(FormControlWithError, { id: "shortName", help: "A less than 10 character name for this bounty. This will be the bounty token's symbol.", label: "Short Name", errors: errors },
                    React.createElement(Input, { ...register("shortName") }))),
                !hide.has("authority") && (React.createElement(FormControlWithError, { id: "authority", help: "The wallet that signs to disburse the funds of this bounty when it is completed. \n            For social tokens, this defaults to the wallet associated with the social token. This\n            can also be an SPL Governance address or a multisig.", label: "Approver", errors: errors },
                    publicKey && (React.createElement(Button, { variant: "link", onClick: () => setValue("authority", publicKey.toBase58()) }, "Set to My Wallet")),
                    React.createElement(Recipient, { name: authorityRegister.name, value: authority, onChange: authorityRegister.onChange }))),
                !hide.has("contact") && (React.createElement(FormControlWithError, { id: "contact", help: "Who to contact regarding the bounty. This can be an email address, twitter handle, etc.", label: "Contact Information", errors: errors },
                    React.createElement(Input, { ...register("contact") }))),
                !hide.has("discussion") && (React.createElement(FormControlWithError, { id: "discussion", help: "A link to where this bounty is actively being discussed. This can be a github issue, forum link, etc. Use this to coordinate the bounty.", label: "Discussion", errors: errors },
                    React.createElement(Input, { ...register("discussion") }))),
                error && React.createElement(Alert, { status: "error" }, error.toString()),
                React.createElement(Button, { type: "submit", alignSelf: "flex-end", colorScheme: "primary", isLoading: isSubmitting || loading, loadingText: awaitingApproval ? "Awaiting Approval" : "Loading" }, "Send Bounty")))));
};
const getFileFromUrl = async (url, name, defaultType = "image/jpeg") => {
    if (!url) {
        return undefined;
    }
    const data = await fetch(url, { cache: "no-cache" });
    const blob = await data.blob();
    const fileName = `${name}${blob.type === defaultType ? ".jpeg" : "png"}`;
    const file = new File([blob], fileName, { type: blob.type || defaultType });
    return file;
};
export const EditBountyForm = ({ mintKey, onComplete, hide = new Set(), }) => {
    const { info: tokenBonding, loading: loadingBonding } = useTokenBondingFromMint(mintKey);
    const { displayName, image, metadata, data, loading: loadingMetadata, } = useTokenMetadata(mintKey);
    const { result: file, loading } = useAsync(getFileFromUrl, [image, "image"]);
    const attributes = React.useMemo(() => SplTokenMetadata.attributesToRecord(data?.attributes), [data]);
    if (loadingBonding ||
        loadingMetadata ||
        loading ||
        !tokenBonding ||
        !metadata) {
        return (React.createElement(Center, null,
            React.createElement(Spinner, null)));
    }
    return (React.createElement(EditBountyFormRaw, { hide: new Set(["contact", "discussion"]), onComplete: onComplete, mintKey: mintKey, values: {
            authority: tokenBonding.reserveAuthority.toBase58(),
            name: displayName,
            image: file,
            description: data?.description,
            shortName: metadata?.data.symbol,
            contact: attributes?.contact,
            discussion: attributes?.discussion,
        } }));
};
//# sourceMappingURL=EditBountyForm.js.map