import { Alert, Button, Flex, Input, Text, VStack } from "@chakra-ui/react";
import { yupResolver } from "@hookform/resolvers/yup";
import { DataV2, Metadata } from "@metaplex-foundation/mpl-token-metadata";
import { useWallet } from "@solana/wallet-adapter-react";
import { useWalletModal } from "@solana/wallet-adapter-react-ui";
import { PublicKey } from "@solana/web3.js";
import { useMint, useProvider, usePublicKey, useStrataSdks, useTokenMetadata } from "@strata-foundation/react";
import { useRouter } from "next/router";
import React, { useEffect } from "react";
import { useAsyncCallback } from "react-async-hook";
import { FormProvider, useForm } from "react-hook-form";
import * as yup from "yup";
import { FormControlWithError } from "./FormControlWithError";
import { MintSelect } from "./MintSelect";
import { TokenMetadataInputs } from "./TokenMetadataInputs";
const validationSchema = yup.object({
    mint: yup.string().required(),
    image: yup.mixed(),
    name: yup.string().required().min(2),
    description: yup.string(),
    symbol: yup.string().min(2).max(10),
    externalUrl: yup.string(),
});
async function editMetadata(tokenMetadataSdk, values) {
    const mintKey = new PublicKey(values.mint);
    const uri = await tokenMetadataSdk.uploadMetadata({
        provider: values.provider,
        name: values.name,
        symbol: values.symbol,
        description: values.description,
        image: values.image,
        mint: mintKey,
        externalUrl: values.externalUrl,
    });
    const instructions = [];
    const signers = [];
    const metadata = await Metadata.getPDA(mintKey);
    const data = new DataV2({
        // Max name len 32
        name: values.name.substring(0, 32),
        symbol: values.symbol.substring(0, 10),
        uri,
        sellerFeeBasisPoints: 0,
        creators: null,
        collection: null,
        uses: null,
    });
    if (await tokenMetadataSdk.provider.connection.getAccountInfo(metadata)) {
        const { instructions: metaInstrs, signers: metaSigners } = await tokenMetadataSdk.updateMetadataInstructions({
            metadata,
            data,
        });
        instructions.push(...metaInstrs);
        signers.push(...metaSigners);
    }
    else {
        const { instructions: metaInstrs, signers: metaSigners } = await tokenMetadataSdk.createMetadataInstructions({
            mint: mintKey,
            data,
        });
        instructions.push(...metaInstrs);
        signers.push(...metaSigners);
    }
    await tokenMetadataSdk.sendInstructions(instructions, signers);
    return mintKey;
}
export const EditMetadataForm = ({ values, onComplete, }) => {
    const formProps = useForm({
        resolver: yupResolver(validationSchema),
        defaultValues: values,
    });
    const { register, handleSubmit, watch, setValue, formState: { errors, isSubmitting }, } = formProps;
    const { awaitingApproval } = useProvider();
    const { execute, loading, error } = useAsyncCallback(editMetadata);
    const { tokenMetadataSdk } = useStrataSdks();
    const { visible, setVisible } = useWalletModal();
    const { connected, publicKey } = useWallet();
    const router = useRouter();
    const { mint, name, symbol } = watch();
    const mintKey = usePublicKey(mint);
    const mintAcc = useMint(mintKey);
    const { metadata, data, image } = useTokenMetadata(mintKey);
    const insufficientAuthority = metadata?.updateAuthority
        ? metadata.updateAuthority != publicKey?.toBase58()
        : false;
    useEffect(() => {
        if (mint) {
            setValue("mint", mint);
        }
    }, [mint, router, setValue]);
    useEffect(() => {
        if (metadata && data && image && !(isSubmitting || loading)) {
            setValue("name", metadata?.data.name);
            setValue("symbol", metadata?.data.symbol);
            setValue("description", data?.description || "");
            setValue("externalUrl", data?.external_url || "");
            (async () => {
                const imageFile = await getFileFromUrl(image, "image");
                if (imageFile)
                    setValue("image", imageFile);
            })();
        }
    }, [setValue, metadata, data, image]);
    const onSubmit = async (values) => {
        await execute(tokenMetadataSdk, values);
        onComplete && onComplete(new PublicKey(values.mint));
    };
    return (React.createElement(Flex, { position: "relative", w: "100%" },
        !connected && (React.createElement(Flex, { position: "absolute", w: "full", h: "full", zIndex: "1", flexDirection: "column" },
            React.createElement(Flex, { justifyContent: "center" },
                React.createElement(Button, { colorScheme: "orange", variant: "outline", onClick: () => setVisible(!visible) }, "Connect Wallet")),
            React.createElement(Flex, { w: "full", h: "full", bg: "white", opacity: "0.6" }))),
        React.createElement(FormProvider, { ...formProps },
            React.createElement("form", { onSubmit: handleSubmit(onSubmit), style: { width: "100%" } },
                React.createElement(VStack, { spacing: 8, w: "full" },
                    React.createElement(FormControlWithError, { id: "mint", help: "The mint/token id of the token whose metadata you wish to update.", label: "Mint", errors: errors },
                        name && (React.createElement(Text, { color: "gray.400", size: "sm" },
                            name,
                            " (",
                            symbol,
                            ")")),
                        React.createElement(MintSelect, { value: watch("mint") || "", onChange: (s) => setValue("mint", s) })),
                    React.createElement(TokenMetadataInputs, { entityName: "Token" }),
                    React.createElement(FormControlWithError, { id: "symbol", help: "A less than 10 character symbol for this token.", label: "Short Name", errors: errors },
                        React.createElement(Input, { ...register("symbol") })),
                    mintAcc && mintAcc.decimals == 0 && (React.createElement(FormControlWithError, { id: "externalUrl", help: "An optional URL that will be rendered in some wallets and explorers", label: "External URL", errors: errors },
                        React.createElement(Input, { ...register("externalUrl") }))),
                    error && React.createElement(Alert, { status: "error" }, error.toString()),
                    React.createElement(Button, { isDisabled: insufficientAuthority, type: "submit", alignSelf: "flex-end", colorScheme: "primary", isLoading: isSubmitting || loading, loadingText: awaitingApproval ? "Awaiting Approval" : "Loading" }, insufficientAuthority ? "You do not hold the update authority" : metadata ? "Update Metadata" : "Create Metadata"))))));
};
const getFileFromUrl = async (url, name, defaultType = "image/jpeg") => {
    if (!url) {
        return undefined;
    }
    const data = await fetch(url, { cache: "no-cache" });
    const blob = await data.blob();
    if (!blob.type.includes("image"))
        return undefined;
    const fileName = `${name}${blob.type === defaultType ? ".jpeg" : ".png"}`;
    const file = new File([blob], fileName, { type: blob.type || defaultType });
    return file;
};
//# sourceMappingURL=EditMetadataForm.js.map