import { useMarketplaceSdk } from "../contexts/marketplaceSdkContext";
import { useAsync } from "react-async-hook";
import Fuse from "fuse.js";
import { useMemo, useState } from "react";
import { useStrataSdks } from "@strata-foundation/react";
import { Metadata, } from "@metaplex-foundation/mpl-token-metadata";
import { chunks, SplTokenMetadata } from "@strata-foundation/spl-utils";
import { useEffect } from "react";
async function getBounties(marketplaceSdk, baseMint) {
    if (marketplaceSdk) {
        return marketplaceSdk.getBounties({ baseMint });
    }
}
async function enrich(shouldEnrich, bountyItems, tokenMetadataSdk) {
    if (shouldEnrich) {
        return Promise.all(bountyItems.map(async (bounty) => {
            const tokenMetadata = (await tokenMetadataSdk?.getMetadata(await Metadata.getPDA(bounty.targetMint))) || undefined;
            const data = await SplTokenMetadata.getArweaveMetadata(tokenMetadata?.data.uri);
            return {
                ...bounty,
                tokenMetadata,
                data,
                attributes: SplTokenMetadata.attributesToRecord(data?.attributes),
            };
        }));
    }
    return bountyItems;
}
const BATCH_SIZE = 20;
const LIMIT = 20;
// Enrich batch size at a time with token metadata. This
// keeps us from hitting metadata for every single bounty, only
// the ones we may need. Batch fetches up to limit results returned
function useEnriched(search, bountyItems, tokenMetadataSdk, batchSize = BATCH_SIZE, limit = LIMIT) {
    if (batchSize > limit) {
        batchSize = limit;
    }
    const [enriched, setEnriched] = useState([]);
    const flat = useMemo(() => enriched.flat(), [enriched]);
    useEffect(() => {
        (async () => {
            if (bountyItems) {
                const batched = chunks(bountyItems, batchSize);
                for (const [index, batch] of batched.entries()) {
                    let nextSet = (await enrich(true, batch, tokenMetadataSdk)).filter(i => i.attributes?.is_strata_bounty);
                    if (search) {
                        nextSet = new Fuse(nextSet, {
                            keys: ["tokenMetadata.data.name", "tokenMetadata.data.symbol"],
                            threshold: 0.2,
                        })
                            .search(search)
                            .map((result) => result.item);
                    }
                    setEnriched((e) => {
                        e[index] = nextSet;
                        return [...e];
                    });
                    if (enriched.flat().length + nextSet.length > limit) {
                        break;
                    }
                }
            }
        })().catch(console.error);
    }, [bountyItems, batchSize, search, tokenMetadataSdk, limit]);
    if (!bountyItems || !tokenMetadataSdk) {
        return undefined;
    }
    return flat;
}
export function useBounties({ search, baseMint, sortType = "GO_LIVE", sortDirection = "DESC", limit = LIMIT, }) {
    const { tokenMetadataSdk } = useStrataSdks();
    const { marketplaceSdk } = useMarketplaceSdk();
    const { result: bounties, ...rest } = useAsync(getBounties, [
        marketplaceSdk,
        baseMint,
    ]);
    const sorted = useMemo(() => {
        if (bounties) {
            return [...bounties.sort((a, b) => {
                    const sortField = sortType === "GO_LIVE"
                        ? (thing) => thing.goLiveUnixTime.toNumber()
                        : (thing) => thing.reserveBalanceFromBonding.toNumber();
                    return sortDirection === "DESC"
                        ? sortField(b) - sortField(a)
                        : sortField(a) - sortField(b);
                })];
        }
        return [];
    }, [bounties, sortType, sortDirection]);
    const enriched = useEnriched(search, sorted, tokenMetadataSdk, BATCH_SIZE, limit);
    return {
        ...rest,
        result: enriched,
    };
}
//# sourceMappingURL=useBounties.js.map